from howsit import parse
from howsit.test import self_test


def main(executor, output, args):
    if '--self-test' in args:
        self_test.self_test(executor, output, args[-1])
        return
    output.write(parse.get_indicator(executor))



import enum

import seashore


@enum.unique
class _Status(enum.IntEnum):
    UNTRACKED = 0
    UNCOMMITTED = 1
    UNPUSHED = 2
    NO_UPSTREAM = 3
    NOT_GIT = 4
    OK = 5  # pragma pylint: disable=invalid-name


_INDICATOR = {
    _Status.UNTRACKED: '!',
    _Status.UNCOMMITTED: 'C',
    _Status.UNPUSHED: 'P',
    _Status.NO_UPSTREAM: 'U',
    _Status.NOT_GIT: 'G',
    _Status.OK: 'K',
}


def get_problems(output):
    lines = output.splitlines()
    for line in lines:
        if line.startswith(b'#'):
            if b'...' not in line:
                yield _Status.NO_UPSTREAM
            if b'[' not in line:
                continue
            line = line.split(b'[')[1]
            if b'ahead' in line:
                yield _Status.UNPUSHED
        elif line.startswith(b'??'):
            yield _Status.UNTRACKED
        else:
            yield _Status.UNCOMMITTED
    yield _Status.OK


def get_indicator(executor):
    try:
        out, _ = executor.git.status(branch=None, porcelain=None).batch()
    except seashore.ProcessError:
        problem = _Status.NOT_GIT
    else:
        problem = min(get_problems(out))
    return _INDICATOR[problem]



import sys

import seashore

from howsit import main

if __name__ != '__main__':
    raise ImportError('this module cannot be imported')

EXECUTOR = seashore.Executor(seashore.Shell())
main.main(EXECUTOR, sys.stdout, sys.argv)



import unittest

import seashore

from howsit import parse


class ProblemTest(unittest.TestCase):

    def test_empty(self):
        output = b""
        lst = list(parse.get_problems(output))
        no_problem = lst.pop(0)
        self.assertEquals(lst, [])
        self.assertEquals(no_problem.name, 'OK')

    def test_no_upstream(self):
        output = b"## v1\n"
        lst = list(parse.get_problems(output))
        no_problem = lst.pop()
        problem = lst.pop(0)
        self.assertEquals(lst, [])
        self.assertEquals(no_problem.name, 'OK')
        self.assertEquals(problem.name, 'NO_UPSTREAM')

    def test_no_push(self):
        output = b"## v1...origin/v1 [ahead 1]\n"
        lst = list(parse.get_problems(output))
        no_problem = lst.pop()
        problem = lst.pop(0)
        self.assertEquals(lst, [])
        self.assertEquals(no_problem.name, 'OK')
        self.assertEquals(problem.name, 'UNPUSHED')

    def test_untracked(self):
        output = b"?? fff\n"
        lst = list(parse.get_problems(output))
        no_problem = lst.pop()
        problem = lst.pop(0)
        self.assertEquals(lst, [])
        self.assertEquals(no_problem.name, 'OK')
        self.assertEquals(problem.name, 'UNTRACKED')

    def test_uncommitted(self):
        output = b" M src/howsit/test/test_parse.py\n"
        lst = list(parse.get_problems(output))
        no_problem = lst.pop()
        problem = lst.pop(0)
        self.assertEquals(lst, [])
        self.assertEquals(no_problem.name, 'OK')
        self.assertEquals(problem.name, 'UNCOMMITTED')


class DummyShell(object):

    is_git = True

    def clone(self):
        return self

    def batch(self, cmd, *args, **kwargs):
        if cmd[:2] == ['git', 'status']:
            if set(cmd[2:]) != set(['--porcelain', '--branch']):
                raise ValueError("unknown status command", cmd, args, kwargs)
            if self.is_git:
                return b'', b''
            else:
                raise seashore.ProcessError(2, '', 'not a git directory\n')
        raise ValueError("unknown command", cmd, args, kwargs)


class IndicatorTest(unittest.TestCase):

    def setUp(self):
        self.shell = DummyShell()
        self.executor = seashore.Executor(self.shell)

    def test_simple_get_indicator(self):
        ret = parse.get_indicator(self.executor)
        self.assertEquals(ret, 'K')

    def test_not_git_get_indicator(self):
        self.shell.is_git = False
        ret = parse.get_indicator(self.executor)
        self.assertEquals(ret, 'G')



"""
Functional test
"""
import os
import sys


def run_me(executor):
    """
    Run ourselves
    """
    howsit = [sys.executable, '-m', 'howsit']
    return executor.command(howsit).batch()


def self_test(executor, output, cwd):
    """
    Functional test function
    """
    executor = executor.chdir(cwd)
    executor.command(['git', 'init']).batch()
    output, error = run_me(executor)
    if error != b'':
        raise ValueError('stderr not empty', error)
    if output != b'U':
        raise ValueError('output not U', output)
    with open(os.path.join(cwd, 'not_in_git'), 'w') as filep:
        filep.close()
    output, error = run_me(executor)
    if error != b'':
        raise ValueError('stderr not empty', error)
    if output != b'!':
        raise ValueError('output not !', output)
    executor.command(['git', 'add', 'not_in_git']).batch()
    output, error = run_me(executor)
    if error != b'':
        raise ValueError('stderr not empty', error)
    if output != b'C':
        raise ValueError('output not C', output)



import setuptools

with open('README.rst') as fp:
    long_description = fp.read()

setuptools.setup(
    name='howsit',
    license='MIT',
    description="Howsit: how bad is git today?",
    long_description=long_description,
    use_incremental=True,
    setup_requires=['incremental'],
    author="Moshe Zadka",
    author_email="zadka.moshe@gmail.com",
    packages=setuptools.find_packages(where='src'),
    package_dir={"": "src"},
    install_requires=['attrs', 'incremental', 'seashore',
                      "enum34 ; python_version < '3.4'"],
)
